create database qryflex;

/* * * * * * *   Q U E R Y F L E X    D I C T I O N A R Y    * * * * * * */


      /* Table information loaded from application dictionary */
   create table dicttable  (
      tabsqlname     char(18),
      tabname        char(30),
      ncols          integer,
      nindexes       integer,
      comment        char(80)
      );
   create unique index dicttabletabid
        on dicttable (tabsqlname);

      /* Column information loaded from application dictionary */
   create table dictcolumn  (
     tabsqlname     char(18),
     colsqlname     char(18),
     colid          serial,
     colname        char(30),
     coltypename    char(10),
     coltype        smallint,
     collength      smallint,
     declength      smallint,
     colformat      char(30),
     heading        char(30),
     comment        char(80)
     );
   create unique index dictcolumnname
        on dictcolumn (tabsqlname, colsqlname);
   create unique index dictcolumncolid
        on dictcolumn (tabsqlname, colid, colsqlname);
   create unique index dictcolumncolname
        on dictcolumn (colsqlname, colid);

   create table dictjoin (
      joinid       serial(100),
      jointype     char(1), /* Relation type: one-to-one, one-to-many */
      tabsqlname_from   char(18),
      tabsqlname_to     char(18),
      colsqlname1_from   char(18),
      colsqlname1_to     char(18),
      colsqlname2_from   char(18),
      colsqlname2_to     char(18),
      colsqlname3_from   char(18),
      colsqlname3_to     char(18),
      colsqlname4_from   char(18),
      colsqlname4_to     char(18),
      colsqlname5_from   char(18),
      colsqlname5_to     char(18),
      colsqlname6_from   char(18),
      colsqlname6_to     char(18),
      colsqlname7_from   char(18),
      colsqlname7_to     char(18),
      colsqlname8_from   char(18),
      colsqlname8_to     char(18)
      );
   create unique index dictjointabid
        on dictjoin (tabsqlname_from, tabsqlname_to);

   create index dictjointabto
        on dictjoin (tabsqlname_to);

   create table dictcoltype  (
        coltypename char(10), 
        coltype     smallint,
        collength   smallint,
        declength   smallint,
        colformat   char(30)
        );
   create unique index dictcoltypename
        on dictcoltype (coltypename);
   create index dictcoltypetype
        on dictcoltype (coltype);
   
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("char",     0, "", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("smallint", 1, "2", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("integer",  2, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("long",     2, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("float",    3, "8", "4"); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("smallfloat", 4, "8", "2"); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("decimal",  5, "8", "4"); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("serial",   6, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("date",     7, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("money",    8, "8", "2"); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("time",     9, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("dtime",   10, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("binary",  11, "", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("mtime",   12, "4", ""); 
insert into dictcoltype (coltypename, coltype, collength, declength) 
        values ("flob",   13, "30", ""); 



/* * * * * * * * *     M E N U     S T R U C T U R E     * * * * * * * * * * */
   /***************************
   MODULE       RELATION        TABLES
   ---------    -------------   ------------
   AR           Invoices        armopen
                                ardopen
                                armcus
                Customer Info   armcus
                                ardcus
                Billing
   GL
   ***************************/

   create table qfmodule  (
      modid         serial(100),
      name          char(30),/*SALES, AR, AP, GL, etc */
      seqno         integer, /* determines order MODULES will appear on qf */
      markseqno     smallint
      );
   create unique index qfmodulemodid
        on qfmodule (modid);
   create unique index qfmoduleseqno
        on qfmodule (seqno, modid);


   create table qfrelation  (
      modid        integer,
      relid        serial(100),
      name         char(30),/* Orders, Customers, etc  */
      seqno        integer, /* determines order RELATION will appear on qf*/
      markseqno    smallint
      );
   create unique index qfrelationrelid
        on qfrelation (modid, relid);
   create unique index qfrelationname
        on qfrelation (modid, name);
   create unique index qfrelationseqno
        on qfrelation (modid, seqno, relid);

   create table qftable  (
      modid        integer,
      relid        integer,
      tabid        serial(100),
      tabsqlname   char(18),
      seqno        integer, /* determines order RELATION will appear on qf*/
      markseqno    smallint
      );
   create unique index qftabletabid
        on qftable (modid, relid, tabsqlname);
   create unique index qftableseqno
        on qftable (modid, relid, seqno, tabid);
   create unique index qftabletabname
        on qftable (tabsqlname, tabid);

   create table qfcolumn  (
      modid        integer,
      relid        integer,
      colid        serial,
      tabsqlname   char(18),
      colsqlname   char(18),
      seqno        integer, /* determines order RELATION will appear on qf*/
      markseqno    smallint
      );
   create unique index qfcolumncolid 
        on qfcolumn (modid, relid, tabsqlname, colsqlname);
   create unique index qfcolumnseqno
        on qfcolumn (modid, relid, tabsqlname, seqno, colid);
   create unique index qfcolumncolserial
        on qfcolumn (colid);






   create table qfreport  (
     modid          integer,
     relid          integer,
     owner          char(18),  /* username or PUBLIC */
     seqno          integer,   /* determines order of reports on qf */
     rptid          serial,
     rptname        char(18),
     createdate     date,
     createtime     time,
     title          char(40),
     comment        char(80)
     );

   create unique index qfreportrptid
        on qfreport (rptid);

   create unique index qfreportrptname
        on qfreport (owner, rptname);

   create unique index qfreportseqno
        on qfreport (modid, relid, owner, seqno, createdate, createtime, rptid);




/* * * * * * * * * * * *     S E C U R I T Y     * * * * * * * * * * * * */



   create table qfuser (
     usercode       char(10),  /* UNIX: use login name, DOS: prompt */
     usertype       char(10),
     password       char(10),
     description    char(25)
     );
  create unique index qfusercode
  on qfuser (usercode);

   create table qfusty (
     usertype       char(10),
     description    char(25)
     );
  create unique index qfustycode
  on qfusty (usertype);

   create table qfperm  (
     usertype    char(10),
     permtype    char(1),
     password    char(10),
     permid      serial,
     modid         integer,
     relid         integer,
     tabid         integer,
     colid         integer
     );
        create unique index  qfpermcolid
        on qfperm (usertype, modid, relid, tabid, colid, permid);

   create table qfpermre  (
     usertype    char(10),
     permtype    char(1),
     password    char(10),
     permid      serial,
     rptid         integer
     );
        create unique index  qfpermrptid
        on qfpermre (usertype, rptid, permid);

